import React, { useEffect, useState } from "react";
import { useThree } from "@react-three/fiber";
import useModuleStore from "../../../store/useModuleStore";
import { useSimulationStates } from "../../../store/store";
import * as SimulationTypes from '../../../types/simulationTypes';
import { ArmbotInstances } from "./ArmBotInstances";

interface ArmBotState {
    uuid: string;
    position: [number, number, number];
    rotation: [number, number, number];
    status: string;
    material: string;
    triggerId: string;
    actions: { uuid: string; name: string; speed: number; processes: { triggerId: string; startPoint: string; endPoint: string }[]; };
}

interface ArmBotProps {
    armBots: ArmBotState[];
    setArmBots: React.Dispatch<React.SetStateAction<ArmBotState[]>>;
}

const ArmBot = ({ armBots, setArmBots }: ArmBotProps) => {
    const { activeModule } = useModuleStore();
    const { scene } = useThree();
    const { simulationStates } = useSimulationStates();

    useEffect(() => {
        const filtered = simulationStates.filter((s): s is SimulationTypes.ArmBotEventsSchema => s.type === "ArmBot");
        const initialStates: ArmBotState[] = filtered.map(bot => ({
            uuid: bot.modeluuid,
            position: bot.position,
            rotation: bot.rotation,
            status: "idle",
            material: "default",
            triggerId: '',
            actions: bot.points.actions
        }));
        setArmBots(initialStates);
    }, [simulationStates]);

    useEffect(() => {
        armBots.forEach((bot) => {
            const object = scene.getObjectByProperty("uuid", bot.uuid);
            if (object) {
                object.visible = activeModule !== "simulation";
            }
        });
    }, [scene, activeModule, armBots]);

    return (
        <>
            {activeModule === "simulation" &&
                armBots.map((bot, i) => (
                    <ArmbotInstances
                        key={i}
                        index={i}
                        armBot={bot}
                        setArmBots={setArmBots}
                    />
                ))}
        </>
    );
};

export default ArmBot;
