import IkInstances from "./IkInstances";
import armModel from "../../../assets/gltf-glb/rigged/ik_arm_4.glb";
import { useEffect, useState } from "react";
import { useThree } from "@react-three/fiber";
import { Vector3 } from "three";

interface Process {
    triggerId: string;
    startPoint?: Vector3;
    endPoint?: Vector3;
    speed: number;
}

interface ArmBotState {
    uuid: string;
    position: [number, number, number];
    rotation: [number, number, number];
    status: string;
    material: string;
    triggerId: string;
    actions: {
        uuid: string;
        name: string;
        speed: number;
        processes: {
            triggerId: string;
            startPoint: string;
            endPoint: string;
        }[];
    };
}

interface ArmbotInstancesProps {
    index: number;
    armBot: ArmBotState;
    setArmBots: React.Dispatch<React.SetStateAction<ArmBotState[]>>;
}

export const ArmbotInstances: React.FC<ArmbotInstancesProps> = ({ index, armBot, setArmBots }) => {
    const { scene } = useThree();
    const [processes, setProcesses] = useState<Process[]>([]);

    useEffect(() => {

        if (armBot.actions.processes.length > 0) {
            const mappedProcesses = armBot.actions.processes.map((process) => {
                return {
                    triggerId: process.triggerId,
                    startPoint: scene.getObjectByProperty('uuid', process.startPoint)?.getWorldPosition(new Vector3()),
                    endPoint: scene.getObjectByProperty('uuid', process.endPoint)?.getWorldPosition(new Vector3()),
                    speed: armBot.actions.speed
                };
            });
            setProcesses(mappedProcesses);
        } else {
            setProcesses([]);
        }
    }, [armBot, scene]);

    const updateArmBotStatus = (status: string) => {
        setArmBots((prevArmBots) => {
            return prevArmBots.map(bot => {
                if (bot.uuid === armBot.uuid) {
                    return { ...bot, status };
                }
                return bot;
            });
        });
    };

    return (
        <IkInstances
            key={index}
            uuid={armBot.uuid}
            modelUrl={armModel}
            position={armBot.position}
            rotation={armBot.rotation}
            processes={processes}
            updateArmBotStatus={updateArmBotStatus}
        />
    );
};